﻿Imports PluginCalculator.Plugins
Imports System.Reflection
Imports System.IO

Public Class MainForm
    Private plugins As New List(Of Plugin)() 'Liste mit allen geladenen Plugins

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.LoadPlugins() 'Plugins laden
    End Sub

    ''' <summary>
    ''' Lädt alle Plugins aus dem Standard Pluginsverzeichnis.
    ''' </summary>
    Private Sub LoadPlugins()
        Dim path As String = GetPluginsDirectory() 'Pluginverzeichnis ermitteln

        For Each file As FileInfo In New DirectoryInfo(path).GetFiles("*.dll") 'Durch .dll-Dateien in Pluginsverzeichnis iterieren
            Try
                For Each plugin As Plugin In Me.GetPlugins(file.FullName) 'Durch instanzierte Plugins iterieren
                    Me.LoadPlugin(plugin) 'Plugin laden
                Next
            Catch ex As Exception 'Bei Ausnahmen Fehlermeldung anzeigen
                MessageBox.Show("Plugin """ & file.Name & """ konnte nicht geladen werden:" & vbNewLine & _
                                "  " & ex.Message, _
                                "Plugin Calculator", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try
        Next
    End Sub

    ''' <summary>
    ''' Lädt ein Plugin in die Pluginliste der Form und in das Operator Auswahlmenü.
    ''' </summary>
    ''' <param name="plugin">Das zu ladende Plugin.</param>
    Private Sub LoadPlugin(ByVal plugin As Plugin)
        If Not Me.plugins.Exists(Function(x As Plugin) x.OperatorName = plugin.OperatorName) Then 'Wenn kein Plugin mit dem gleichen Operator bereits geladen ist
            Me.plugins.Add(plugin) 'Plugin zu Liste hinzufügen
            Me.OperatorComboBox.Items.Add(plugin.OperatorName) 'Operator zu Dropdown-Menü hinzufügen

            Me.CalculateButton.Enabled = True 'Berechnen-Button aktivieren, da jetzt mindestens ein Plugin geladen
            Me.OperatorComboBox.SelectedIndex = 0 'Ersten Operator auswählen
        Else 'Wenn Plugin mit gleichem Operator bereits geladen, Fehlermeldung anzeigen
            MessageBox.Show("Ein Plugin mit dem Operator """ & plugin.OperatorName & """ ist bereits geladen.", _
                            "Plugin Calculator", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End If
    End Sub

    ''' <summary>
    ''' Instanziert alle Plugins aus einer Pluginassembly.
    ''' </summary>
    ''' <param name="path">Der Pfad zur Pluginassembly.</param>
    Private Shared Function GetPlugins(ByVal path As String) As List(Of Plugin)
        Dim ass = Assembly.LoadFile(path) 'Assembly (DLL) laden
        Dim plugins As New List(Of Plugin)() 'Liste mit allen geladenen Plugins

        For Each tp As Type In ass.GetTypes() 'Durch alle Typen (Klassen) in der Assembly iterieren
            If tp.GetInterfaces().Contains(GetType(Plugin)) Then 'Wenn Typ Plugin implementiert
                Dim instance = DirectCast(Activator.CreateInstance(tp), Plugin) 'Plugin instanzieren
                plugins.Add(instance) 'Plugin zu Liste hinzufügen
            End If
        Next

        Return plugins 'Liste zurückgeben
    End Function

    ''' <summary>
    ''' Gibt das Pluginstandardverzeichnis zurück
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shared Function GetPluginsDirectory() As String
        Dim path = IO.Path.Combine(Application.CommonAppDataPath, "plugins") 'Pfad zusammenstellen
        If Not Directory.Exists(path) Then Directory.CreateDirectory(path) 'Ggf. Verzeichnis erstellen
        Return path 'Pfad zurückgeben
    End Function

    Private Sub InstallPluginButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InstallPluginButton.Click
        If Me.OpenFileDialog.ShowDialog() = DialogResult.OK Then 'Wenn im Dateiauswahldialog auf OK geklickt
            Try
                GetPlugins(Me.OpenFileDialog.FileName) 'Assembly laden und Plugins instanzieren, um Assembly zu validieren

                Dim targetPath = Path.Combine(GetPluginsDirectory(), Path.GetFileName(Me.OpenFileDialog.FileName)) 'Zielpfad des Plugins

                Dim i As Integer = 2
                Do While File.Exists(targetPath) 'Wenn Zieldatei bereits existiert, hinter Dateinamen Zähler einfügen und hochzählen Bsp: meinplugin (2).dll
                    targetPath = Path.Combine(GetPluginsDirectory(), Path.GetFileNameWithoutExtension(Me.OpenFileDialog.FileName)) & " (" & i.ToString() & ").dll"
                    i += 1
                Loop

                File.Copy(Me.OpenFileDialog.FileName, targetPath) 'Datei kopieren
                Dim plugins = GetPlugins(targetPath) 'Plugins instanzieren
                For Each plugin As Plugin In plugins 'Plugins laden
                    Me.LoadPlugin(plugin)
                Next
            Catch ex As Exception 'Bei Ausnahmen Fehlermeldung anzeigen
                MessageBox.Show("Plugin konnte nicht installiert werden:" & vbNewLine & _
                                "  " & ex.Message, _
                                "Plugin Calculator", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try
        End If
    End Sub

    Private Sub TextBox_Leave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles XTextBox.Leave, YTextBox.Leave
        Dim tb = DirectCast(sender, TextBox) 'Sender auf TextBox casten
        If tb.Text = "" Then tb.Text = "0" 'Wenn TextBox leer, Text auf 0 setzen
        If Not Double.TryParse(tb.Text, 0) Then 'Wenn Inhalt keine gültige Zahl
            tb.Focus() 'TextBox wieder fokussieren
            Beep() 'Signalton ausgeben
            tb.SelectAll() 'TextBox-Inhalt markieren
        End If
    End Sub

    Private Sub CalculateButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CalculateButton.Click
        Dim x, y As Double
        If Not Double.TryParse(Me.XTextBox.Text, x) OrElse Not Double.TryParse(Me.YTextBox.Text, y) Then Return 'Inhalte der TextBoxen parsen; wenn Inhalt ungültig, abbrechen

        Try
            Me.ResultLabel.Text = Me.plugins(Me.OperatorComboBox.SelectedIndex).Calculate(x, y).ToString() 'Berechnung durchführen
        Catch ex As Exception 'Bei Ausnahmen Fehlermeldung anzeigen
            MessageBox.Show("Berechnung konnte nicht durchgeführt werden:" & vbNewLine & _
                                "  " & ex.Message, _
                                "Plugin Calculator", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub
End Class
